function a = agent_initlearn(a, info)
% Initializes the agent's learning behaviour
%  A = AGENT_INITLEARN(A, INFO)
%  Initializes the agent's learning behaviour. This function must be called
%  prior to any calls to agent_control.
%
%  Parameters:
%   A           - the agent
%   INFO        - a structure containing other parameters that
%               the agent might use during learning initialization.
%
%  The INFO structure must contain at least the following fields:
%  'episodic'       - a flag signaling whether the task is episodic
%  'nagents'        - the number of agents in the task
%  'statespacesize' - the size of the world state space
%  'actionspacesize'- the size of the joint action space
%  'imaps'          - a column vector specifying where in the world state
%  vector the agent finds its own state and the common state of the world
%  (i.e. state elements independent of other agents), and where on the
%  joint action and reward vectors the agent finds its own action and
%  reward.
%
%  For the structure of these fields, see world_info.
%
%  This function calls, in sequence:
%  1. the actual learning initialization function of the agent.
%  2. the 'init' mode of the action function, supplying the INFO structure
%  on the parameters.
%  3. the 'init' mode of the exploration function.
%
%  These functions/modes will have access to the 'indices' and 'learnparam'
%  fields of the agent.
%
%  Usually, this function needs not be called directly, but is handled by
%  the learning control mechanism.
%
%
%  See also agent, agent_control

%  Author:      Lucian Busoniu
%  Version:     1.0
%  History:

a.indices = info.imap;
a.k = 0;
a.learnparam.episodic = info.episodic;
if info.episodic,
    a.trialsk = 0;
    % obviously, the task is not finished apriori
    a.finished = 0;
end;
a = feval(strcat(a.learnfun, '_init'), a, info);
a = feval(a.actfun, a, [], [], [], struct('mode', 'init', 'info', info));
a = feval(a.explorefun, a, [], [], [], [], struct('mode', 'init', 'info', info));

% returning a
% END agent_initlearn() ===================================================