function a = teamq(a, state, action, reward, params)
%Implements the team Q-learning algorithm
%  A = TEAMQ(A, STATE, ACTION, REWARD, PARAMS)
%  Implements team Q-learning (also known as friend-Q). This is a
%  straightforward extension of plain Q to the multiagent case. Each agent
%  learns a Q-table on the basis of the full world state and the joint
%  action.
%
%  Required values on the agent learning parameters:
%   alpha           - the learning rate
%   gamma           - the discount factor
%   lambda          - the eligibility trace decay rate
%   epsilon         - the exploration probability
%  Required values on the extra parameters:
%   newtrial        - only in episodic environments; whether a new trial is
%                   beginning
%
% Supports discrete states and actions, with 1 action variable per agent.
%
% Can be coupled with an action function that uses a Q-table indexed on
% full world state and joint action, such as fullstatejoint_greedyact().
%
%
%  References:
%  [1] Littman, M. L. (2001). Friend-or-foe Q-learning in general-sum
%      games. In Proceedings of the Eighteenth International Conference on
%      Machine Learning (ICML-01), pages 322-328, Williams College,
%      Williamstown Massachusets, USA.
%
%  See also agent_learn, teamq_init

%  Author:      Lucian Busoniu
%  Version:     1.1
%  History:
%   - 12 Jan 2006, changed to use monolithic full state instead of just
%   single-agent state
%   - 6 Sept 2006, optimized speed when no elig trace is used


% by convention, all idle agents "take" deterministically their
% first action, so that learning is simplified for other agents
% Note that in a theoretical fully cooperative setting, there will be no
% idle agents
action(isnan(action)) = 1;          % use logical indexing

% decay / compute learning rates
if length(a.learnparam.alphadecay) == 1,
    a.learnparam.alpha = a.learnparam.alpha * a.learnparam.alphadecay;
else
    a.learnparam.alpha = 1 / (a.learnparam.alphadecay(1) + a.k / a.learnparam.alphadecay(2));
end;

if a.useelig && a.learnparam.episodic && params.newtrial, 
    a.E = 0 * a.Q;          % clear eligibility trace
end;

% convert both old index and start index of new state at once
qix = ndi2lin([action' a.state'; ones(1, length(action)) state'], a.sizes.q);
delta = reward(a.indices(7)) + a.learnparam.gamma * max(a.Q(qix(2) : qix(2) + a.sizes.expact - 1)) - a.Q(qix(1));
if a.useelig,
    a.E(qix(1)) = 1;
    a.Q = a.Q + a.learnparam.alpha * delta * a.E;
    a.E = a.learnparam.gamma * a.learnparam.lambda * a.E * ~a.exploring;
else        % just one element gets updated
    a.Q(qix(1)) = a.Q(qix(1)) + a.learnparam.alpha * delta;
end;
    
% returning a
% END teamq() ============================================================
