function a = wolfq(a, state, action, reward, params)
%Implements the Win-or-Learn-Fast Policy Hill Climbing algorithm
%  A = WOLFQ(A, STATE, ACTION, REWARD, PARAMS)
% Implements the Win-or-Learn-Fast Policy Hill Climbing algorithm [1].
% While the originally presented algorithm assumes the whole monolithic
% state of the world is used by the agents, this version also supports
% learning also on the basis of the single agent's state (flag
% "usefullstate" on the learning parameters).
% Here, agents only uses their own state.
%
%  Values on the agent learning parameters:
%   gamma           - the discount factor
%   alpha           - the learning rate
%   alphadecay      - the decay ratio for the learning rate
%   deltaw          - the initial deltaw (for simple delta decay schedule)
%   deltadecay      - the decay ratio for the policy learning rate
%   deltaratio      - how much bigger is the learning rate when losing than
%                   when winning; deltal = deltaratio * deltaw
%   lambda          - eligibility trace decay rate
%   usefullstate    - whether to use the full state in learning
%
% Any decay ratio may be either a single value (in which case a simple
% decay schedule based on plain multiplication is assumed) or a vector of
% two values [a b] in which case Pk = 1 / (a + k/b), where Pk is the value
% of the parameter at the k-th iteration.
%
% Supports discrete states and actions, with 1 action variable per agent.
%
%  References:
%  [1] Bowling, M. and Veloso, M. (2002). Multiagent learning using a
%      variable learning rate. Journal of Artificial Intelligence,
%      136(2):215-250.
%
% See also agent_control, wolfq_init

%  Author:      Lucian Busoniu
%  Version:     1.1
%  History:
%       1.1 added full state


% decay / compute learning rates
if length(a.learnparam.alphadecay) == 1,
    a.learnparam.alpha = a.learnparam.alpha * a.learnparam.alphadecay;
else
    a.learnparam.alpha = 1 / (a.learnparam.alphadecay(1) + a.k / a.learnparam.alphadecay(2));
end;
if length(a.learnparam.deltadecay) == 1,
    a.learnparam.deltaw = a.learnparam.deltaw * a.learnparam.deltadecay;
else
    a.learnparam.deltaw = 1 / (a.learnparam.deltadecay(1) + a.k / a.learnparam.deltadecay(2));
end;
a.learnparam.deltal = a.learnparam.deltaratio * a.learnparam.deltaw;


% clear eligibility trace upon new trial
if a.learnparam.episodic && params.newtrial, 
    a.E = 0 * a.Q;
end;

% helper variables
m = a.actionscount; aspan = 0 : m - 1;
prevstate = a.state(a.xindices);
% next state (s') and action taken to get there
nstate = state(a.xindices); action = action(a.indices(5));

% index to update in Q-table given by previous state and action taken then
qix = ndi2lin([action prevstate'], a.sizes.q);

% find out the best achievable Q-value in the next state
ix = ndi2lin([1 nstate'], a.sizes.q) + aspan;
topQ = max(a.Q(ix));

% do the TD(lambda) update
delta = reward(a.indices(7)) + a.learnparam.gamma * topQ - a.Q(qix);
a.E(qix) = 1;
a.Q = a.Q + a.learnparam.alpha * delta * a.E;
a.E = a.learnparam.gamma * a.learnparam.lambda * a.E * ~a.exploring;

% increment the state counter
% Ci(s) = Ci(s) + 1
ix = ndi2lin(prevstate', a.sizes.c); a.C(ix) = a.C(ix) + 1;

% update average policy estimates
% PIavg_i(s,a') = PIavg_i(s,a') + [PIi(s,a') - PIavg_i(s,a')]/C'(s),
%       for all a' in Ai

% retrieve current and average policies for current state
piix = ndi2lin([1 prevstate'], a.sizes.pi) + aspan;
sPI = a.PI(piix); sPIavg = a.PIavg(piix);

% update average policy estimate
sPIavg = sPIavg + (sPI - sPIavg) / a.C(ix);
a.PIavg(piix) = sPIavg;

% update policy
%
% PIi(st, ai) += Delta_st,ai, ai = 1..|Ai|, where:
% Delta_st,ai = f(delta_i), with:
% delta_i = delta_w, if agent i is winning, delta_l, otherwise
% f ensures the validity of the prob distribution (see comments)

% extract (updated) Q-table slice for current (previous) state
ix = ndi2lin([1 prevstate'], a.sizes.q) + aspan;
piQ = a.Q(ix);

% find out whether winning or losing, and set delta correspondingly
winning = sPI' * piQ > sPIavg' * piQ;
d = winning * a.learnparam.deltaw + ~winning * a.learnparam.deltal;

% find out which actions would now be optimal
opt = find(piQ == max(piQ)); subopt = find(piQ ~= max(piQ));
% how many of the actions are optimal
optcnt = length(opt);
% optmap = zeros(m, 1); optmap(optimal) = 1; 
                
% if all actions are valued equally, there's nothing to change
if optcnt < m,
	% each subopt action is penalized by at most delta/#(subopt)
	d = -min(repmat(d / (m - optcnt), m, 1), sPI);
    % optimal actions are equally rewarded with what quantity of
    % probability is available
    d(opt) = -sum(d(subopt)) / optcnt;
	% change the policies
	a.PI(piix) = sPI + d;
end;        % END policy updating IF


% returning a
% END wolf() =============================================================
