function [world, agents, stats] = episodic_learn(world, agents, learnparam)
%Implements the learning control mechanism in an episodic environment
%  [WORLD, AGENTS, STATS] = EPISODIC_LEARN(WORLD, AGENTS, LEARNPARAM)
%  Performs reinforcement learning over a given number of trials until
%  convergence. A trial is complete when all agents have finished their
%  tasks or after the iterations count has reached a specified upper limit.
%  Learning statistics are maintained. 
%
%  Parameters:
%   WORLD       - the world within which the learning takes place.
%   AGENTS      - the (possibly heterogeneous) cell array of agents.
%               Must be the same with which the world was created.
%   LEARNPARAM  - the learning params. Contains the following fields, all
%               optional.
%               'trials' - how many trials to run at most. Default 100.
%               'maxiter' - how many iterations to run in a trial, at most.
%                   Default 1000.
%               'convgoal' - the convergence goal - the target mean
%                   variation in the number of steps taken for each trial
%                   Default: -1, means the max number of trials will always
%                   run.
%               'avgtrials' - over how many trials to average when checking for 
%                   convergence. Default 10.
%               'showconv'  - whether to plot the convergence progress "live".
%                   Default 0 (do not show).
%               'closeconv' - whether to close convergence progress figure
%                   upon finishing learning.
%  Returns:
%   WORLD       - the possibly altered world.
%   AGENTS      - the (savvier) agents.
%   STATS       - the learning statistics.
%
%  If a convergence goal is specified, learning is stopped when the agents
%  complete 'avgtrials' successive trials with less than 'convgoal'
%  stdev in the iterations taken to complete them. This only makes sense in
%  deterministic environments and with a deterministic initial state of the
%  world at the beginning of each trial.
%
%  The learning statistics are returned in a structure with the following
%  fields:
%   'trials'      - how many trials were run until convergence (or limit)
%   'iter'        - how many iterations each trial took to complete;
%               vector running over trials
%   'elapsedtime' - how much CPU time learning took
%   'worldstats'  - specific statistics maintained by the world; cell vector
%               running over trials (if the world supports this)
%   'syntheticworldstats'
%                 - specific statistics maintained by the world for the
%               entire learning process (if the world supports this)
%
%
%  See also learn, agent_initlearn, agent_act, agent_learn

%  Author:      Lucian Busoniu
%  Version:     1.2
%  Last update: 2006-06-23
%  History:     1.1     - added continuous variables support, world statistics
%               1.2     - added defaults for all params, enabled conv show
%               configurable

LP_DEF.trials = 100;
LP_DEF.maxiter = 1000;
LP_DEF.convgoal = -1;
LP_DEF.avgtrials = 10;
LP_DEF.showconv = 0;
LP_DEF.closeconv = 1;
LP_DEF.convfighandle = 899;     % this should typically not be overwritten
learnparam = checkparams(learnparam, LP_DEF);

% shorthand constants for speedup
wresetfun = strcat(world.type, '_reset');
winfofun = strcat(world.type, '_info');
wdynamicsfun = strcat(world.type, '_dynamics');
wstatsfun = strcat(world.type, '_stats');
statsfunexists = (exist(wstatsfun) ==2);

% retrieve world info
info = feval(winfofun, world);

nagents = length(agents);
nactions = size(info.actionspacesize, 1);           % how many action variables in total
actspans = cell(nagents, 1);                        % positions on the joint action

% setup the agents for learning
ainfo = info; rmfield(ainfo, 'imaps');              % imaps specific to each agent
ainfo.nagents = nagents;                            % add to info: number of agents
ainfo.episodic = 1;                                 % type of task
for i = 1:nagents,
    actspans{i} = info.imaps(5, i) : info.imaps(6, i);
    ainfo.imap = info.imaps(:, i);
%     ainfo.agentindex = i;                         % unsure if this is needed
    agents{i} = agent_initlearn(agents{i}, ainfo);
end;

% initialize loop data and stats
aspan = 1:nagents;
iterspan = 1:learnparam.maxiter;

stats.iter = zeros(1, learnparam.trials);
if statsfunexists, stats.world = cell(1, learnparam.trials); end;

tic;                                                % keep running time stats
for k = 1:learnparam.trials,
%     disp(['Trial ' num2str(k)]);
    action = zeros(nactions, 1);                    % joint action
    actionviews = nan(nactions, nagents);           % undefined last action views
    rewardviews = nan(nagents, nagents);            % undefined last reward views
    finished = zeros(1, nagents);                   % initially noone finished
    % world gets reset at each trial
	[world, stateviews] = feval(wresetfun, world, (k == 1));  
    
    params.donotlearn = 0;
    params.newtrial = 1; j = 0;
    for j = iterspan,
		% controllers evolve first
		for i = aspan,
            params.finished = finished(i);
            [agents{i} action(actspans{i})] = agent_control(agents{i}, ...
                stateviews(:, i), actionviews(:, i), rewardviews(:, i), params);
		end;
        % terminate trial if all finished (but feedback was already given
        % to the agents)
        if finished, break; end;
        % plant evolves
        [world, stateviews, actionviews, rewardviews, finished] = feval(wdynamicsfun, world, action);
        params.newtrial = 0; j = j + 1;
    end;
    
    % update stats
    stats.iter(k) = j - 1;      % the last j corresponds to last feedback to the agents
    if statsfunexists,
        [world stats.worldstats{k}] = feval(wstatsfun, world, 'trial');
    end;
    
    % convergence test, if activated
    if learnparam.convgoal >= 0 && k >= learnparam.avgtrials ...
            && std(stats.iter(k-learnparam.avgtrials+1:k)) < learnparam.convgoal,
        break;
    end;

     % live convergence progress is enabled
     if learnparam.showconv && ~mod(k, 2),
         if k == 2,
             % move world to the left of screen
             if isfield(world, 'view') && isfield(world.view, 'gui') && ishandle(world.view.gui),
                 movegui(world.view.gui, 'west');
             end;
             % need to create figure
             fh = learnparam.convfighandle;
             set(0, 'Units', 'Pixels'); screensize = get(0, 'ScreenSize');
             guih = get(0, 'CurrentFigure');     % any GUI the world is displaying
             figure(fh); figsize = get(fh, 'Position');
             if ~isempty(guih),
                 guisize = get(guih, 'Position');
                 % ensure GUI doesn't cover figure, acknowledging resolution and figure borders
                 figsize(3) = min(figsize(3), screensize(3) - guisize(3) - 50);
                 set(fh, 'Position', figsize);
             end;
             movegui(fh, 'east'); 
         end;
         figure(learnparam.convfighandle); stairs(stats.iter(1:k), 'k', 'LineWidth', 2); grid;
         xlabel('Learning trial'); ylabel('Steps to complete trial'); title('Learning convergence');
     end;
end;      % END trial

stats.elapsed_time = toc;                           % record learning time
% tune and finalize stats
stats.trials = k;
stats.iter = stats.iter(1:k);
if statsfunexists,
    stats.syntheticworldstats = feval(wstatsfun, world, 'synthetic');
end;

if learnparam.showconv && learnparam.closeconv,
    close(learnparam.convfighandle);
end;

% returning world, agents, stats

% END learn() =============================================================