function gw = gridworld(agents, name, worldsize, goalpos, initpos, obstacles, rewards)
%Creates a 2D grid world
%  GW = GRIDWORLD(AGENTS, NAME, WORLDSIZE, GOALPOS, [INITPOS, OBSTACLES, REWARDS])
%  Parameters:
%   AGENTS      - the agents in the gridworld, as a (possibly
%               heterogeneous) cell array
%   NAME        - the name of the gridworld
%   WORLDSIZE   - the size of the 2D gridworld, a vector [W H]
%   GOALPOS     - the goal positions of the agents. One agent position on
%               each row. Should specify goal positions for the maximum
%               number of agents in the gridworld.
%   INITPOS     - (optional) the deterministic initial positions of the
%               agents. If an empty matrix, at each trial start agents will
%               be placed in random positions in the world. One agent
%               position on each column. Should specify initial positions for
%               the maximum number of agents in the gridworld.
%               Default: empty matrix.
%   OBSTACLES   - (optional) the obstacles to place in the gridworld.
%               May be: a matrix with obstacle 2D coordinates on each column;
%               an empty matrix, in which case no obstacles are placed; or
%               the string 'user' to obtain the obstacles interactively
%               from the user. Default is empty matrix (no obstacles)
%   REWARDS     - (optional) a vector with three elements, [RMOVE RCOLL RGOAL],
%               specifying how much should the agents be rewarded /
%               penalized upon an ordinary move, a collision and reaching
%               the goal. Default is [-1 -2 10]
%
%  Agents can move one cell in either of the four compass directions,
%  or stay still. Each agent has one action variable available, and its
%  values are coded in the following way:
%   1 - left
%   2 - right
%   3 - up
%   4 - down
%   5 - stay put
%  Note that an agent can function without the fifth action.
%
%  The agents cannot move out of the grid, onto an obstacle, or onto a goal
%  that does not belong to them. If agents collide, they will be reset to
%  their positions previous to the collision.
%  Each agent finishes when it has reached its corresponding goal. There is
%  no failure situation.
%
%
%  See also gridworldview, gridworld_advance, gridworld_destroy,
%      gridworld_obstacles, gridworld_reset

%  Author:      Lucian Busoniu
%  Version:     1.0
%  History:

REQ_ARG_COUNT = 4; DEFAULT_REWARDS = [-1 -2 10];

% verification of required arguments
if nargin < REQ_ARG_COUNT, error('Incorrect number of inputs'); end;

worldsize = worldsize'; goalpos = goalpos';     % to row format


% world constants initialization
gw.name = name; gw.size = worldsize;
% action displacements in the 2D space of the grid - left, right, up, down,
% stay put
gw.actoffsets = [-1 0; 1 0; 0 -1; 0 1; 0 0]';
% grid world constants
gw.tasktype = 'episodic';
gw.type = 'gridworld';
gw.viewtype = 'gridworldview';

% handle variable argument lists, transforming into the row format
if nargin < REQ_ARG_COUNT + 1, gw.initpos = []; 
else gw.initpos = initpos'; end;
if nargin < REQ_ARG_COUNT + 2, obstacles = []; end;
if nargin < REQ_ARG_COUNT + 3, gw.rewards = DEFAULT_REWARDS;
else gw.rewards = rewards;
end;

gw.grid = zeros(worldsize);

gw.acount = length(agents);
% maintain a shortcut to the world state: agent positions on columns
gw.state = zeros([2 gw.acount]);
% agent ids map specifies the id for an agent on a given position in the
% state matrix
gw.aidmap = zeros([gw.acount 1]);

for i = 1:gw.acount,
    a = agents{i};
    gw.grid(goalpos(i, 1), goalpos(i, 2)) = 1000 + a.id;
    gw.aidmap(i) = a.id;
end;

% if deterministic initial positions, init them now so that obstacles
% cannot be placed on top
if ~isempty(gw.initpos),
    for i = 1:gw.acount,
        gw.grid(gw.initpos(i, 1), gw.initpos(i, 2)) = agents{i}.id;
        gw.state(:, i) = gw.initpos(i, :)';
    end;
    gw.initialstate = gw.state;
else gw.state = ones(2, gw.acount);     % just a placeholder
end;

% place any specified obstacles
if isnumeric(obstacles) && ~isempty(obstacles),
    obstacles = obstacles';             % to row format
    for i = 1 : size(obstacles, 1),
        gw.grid(obstacles(i, 1), obstacles(i, 2)) = -1;
    end;
end;

gw.view = gridworldview(gw);

if ischar(obstacles), if obstacles == 'user',
    % get obstacles interactively
    gw = gridworld_obstacles(gw);
end; end;

% returning gw

% END gridworld()
% =======================================================================