function gw = gridworld_display(gw, visible, update)
%Shows or hides the grid world view
%  V = GRIDWORLD_DISPLAY(GW[, VISIBLE[, UPDATE]])
%  Shows or hides the grid word view. If the UPDATE argument is present, the
%  view is updated to be consistent with the world state. If on the other hand
%  the argument is not present, but the view needs to be shown and knows it is
%  tainted (not consistent with the state of the world) an automatic complete
%  update will be performed.
% 
%  Parameters:
%   GW          - the grid world
%   VISIBLE     - (optional) whether the view should be shown (1) or
%               hidden (0). By default, the view will be shown.
%   UPDATE      - (optional) if present, specifies that the view should be
%               updated for consistency with the world state. May be either
%               the string 'all', in which case all the grid is updated, or
%               a vector of bounds [Xmin Xmax Ymin Ymax], in which case
%               only the specified area (including edges) is updated.
%  Returns:
%   GW          - the updated gridworld
%
%
%  See also gridworldview, gridworld_advance

%  Author:      Lucian Busoniu
%  Version:     1.0
%  History:


% verification of required arguments, defaults for not supplied
REQ_ARG_COUNT = 1;
if nargin < REQ_ARG_COUNT, error('Incorrect number of inputs'); end;
v = gw.view;
if nargin < REQ_ARG_COUNT + 1, visible = 1; end;
if nargin < REQ_ARG_COUNT + 2,  % update all if tainted and showing
    if visible && v.tainted, update = 'all';
    else update = 0; end;
end;

% process update
if update ~= 0,
    if ischar(update) update = [1 v.width 1 v.height]; end;
    for i = update(1):update(2),
        for j = update(3):update(4),
            e = gw.grid(i, j);
            switch gw.grid(i, j),
                  case 0,       % empty cell
                      set(v.polys(i, j), 'FaceColor', v.colors.BLANK, 'EdgeColor', v.colors.BLANK);
                  case -1,      % obstacle cell
                      set(v.polys(i, j), 'FaceColor', v.colors.OBSTACLE, 'EdgeColor', v.colors.OBSTACLE);
                  otherwise,    % agent or goal
                      if e < 1000,      % agent; goals cannot move, so we do not consider them
                          % get the color row containing the agent color
                          colorrow = find(v.colors.AGENTS(:, 1) == e);
                          color = v.colors.AGENTS(colorrow, 2:4);
                          set(v.polys(i, j), 'FaceColor', color, 'EdgeColor', color);
                      end;
              end;      % END cell type switch
          end;      % FOR rows iterator
      end;      % FOR columns iterator
      v.tainted = 0;    % view is up to date
end;        % END process update

% show / hide
if visible,
    set(v.gui, 'Visible', 'on');
    pause(0.003);       % yield some processor for view painting
    % in Windows, increase the pause duration
    % if the gridworld painting engine "hangs" during updates
else
    set(v.gui, 'Visible', 'off');
end;

% returning gw

% END gridworldview_display() =============================================