function [gw, stateviews, actionviews, rewardviews, finished] = gridworld_dynamics(gw, actions)
%Implements the discrete-time dynamics of the gridworld 
%  [GW, STATEVIEWS, ACTIONVIEWS, REWARDVIEWS, FINISHED] = GRIDWORLD_DYNAMICS(GW, ACTIONS)
%  Transforms the state of the world based on the agent actions, and
%  computes the instantaneous rewards. This implementation of the gridworld
%  will always return true and complete views over state, joint actions and
%  joint rewards.
%
%  Parameters:
%   GW          - the gridworld.
%   ACTIONS     - the joint action vector of the agents, a column vector.
%               May contain NaN elements, signifying 'noop' for agents that
%               completed their task.
%
%  Returns:
%   GW          - the altered gridworld.
%   STATEVIEWS  - the views of the agents over the state, nth agent view on
%               the nth column.
%   ACTIONVIEWS - the views of the agents over the joint action, nth agent
%               view on the nth column.
%   REWARDVIEWS - the views of the agents over the joint reward, nth agent
%               view on the nth column.
%   FINISHED    - a boolean row vector containing for each agent a flag 
%               that signals if that agents' task has finished in this trial
%               (either by success, or failure). nth agent flag on the nth
%               column.
%
%  The agents cannot move out of the grid, onto an obstacle, or onto a goal
%  that does not belong to them. Furthermore, if several agents try to move
%  into the same position, they will be reset to their original positions
%  (including any agents that wouldn't have hit an agent if that agent
%  wouldn't have hit something else and been reset etc., recursively).
%
%  Each agent finishes when it has reached its corresponding goal. There is
%  no failure situation.
%
%
%  See also gridworld

%  Author:      Lucian Busoniu
%  Version:     1.0
%  History:


acount = gw.acount;

actions = actions';         % gridworld implemented long ago, with row action standard
% find out which agents tried to act
acting = find(~isnan(actions));
still = find(actions == 5);
moved = zeros(1, acount); putback = moved; atgoal = moved;
moved(acting) = 1; moved(still) = 0;

% construct the candidate state
state = gw.state; cstate = state;
for i = acting,
    castate = cstate(:, i) + gw.actoffsets(:, actions(i));  % candidate state for agent
    % agent can only try to move onto own goal, empty space or another agent
    if sum(castate >= [1;1] & castate <= gw.size') == 2,    % check for out-of-grid
        e = gw.grid(castate(1), castate(2));                % current destination content
        if e >= 0 && (e < 1000 || e == 1000 + gw.aidmap(i)),
% Uncomment if the agents should not be allowed to switch positions. Not
% recommended
%         if (e == 0) || (e == 1000 + gw.aidmap(i)),
            cstate(:, i) = castate;
            atgoal(i) = e > 1000;
        else putback(i) = 1;
        end;
    else putback(i) = 1;
    end;
end;

% send back agents that try to switch positions "passing one through
% another"
movedix = find(moved & ~putback);
for ii = 1 : length(movedix) - 1,
    i = movedix(ii);
    % check whether destination cell holds an agent
    e = gw.grid(cstate(1, i), cstate(2, i));
    if e == 0 || e > 1000, continue; end;
    % if agent, test whether switching positions
    if all(cstate(:, i) == state(:, e)) && all(cstate(:, e) == state(:, i)),
        % put them back
        cstate(:, [i e]) = state(:, [i e]);
        putback([i e]) = 1;
    end;
end;


% send any agents that bumped into each other back to their original positions
i = 1;
while i < acount,
    overlap = find(sum(abs(cstate(:, i+1:acount) - repmat(cstate(:, i), 1, acount - i))) == 0);
    if isempty(overlap),
        i = i + 1; continue;
    else
        overlap = [0 overlap] + i;                  % current agent also
        cstate(:, overlap) = state(:, overlap);     % send agents back
        putback(overlap) = 1;
        i = 1;                                      % reset search
    end;
end;

% still agents have not been "put back" since they did not depart in the
% first place
putback(still) = 0;
% reset agents were those that were trying to move and were put back
reset = find(moved & putback);
% the ones that departed their original positions were those that weren't put back
departed = find(moved & ~putback);
% agents that reached their goal dissappear into nothingness from the grid
arrived = find(moved & ~putback & ~atgoal);


% change the positions of the moved agents in the grid
grid = gw.grid(:);
if ~isempty(departed), grid(sub2ind(gw.size, state(1, departed), state(2, departed))) = 0; end;
if ~isempty(arrived), grid(sub2ind(gw.size, cstate(1, arrived), cstate(2, arrived))) = gw.aidmap(arrived); end;
gw.grid(:) = grid;

% promote the (now consistent) candidate state
gw.state = cstate;

% distribute rewards to agents
rewards = gw.rewards(1) * ones(acount, 1); rewards(reset) = gw.rewards(2); rewards(find(atgoal)) = gw.rewards(3);

% generate views for all agents
stateviews = repmat(cstate(:), 1, acount);
actionviews = repmat(actions', 1, acount);      % back to column format
rewardviews = repmat(rewards, 1, acount);

% agents are finished if they were finished before, or they reached the goal
finished = atgoal | isnan(actions);

% repaint or schedule for repaint depending on visibility state
if gridworldview_isvisible(gw.view),    % update view
    xs = [state(1, departed) cstate(1, arrived)]; ys = [state(2, departed), cstate(2, arrived)];
    gw = gridworld_display(gw, 1, [min(xs) max(xs) min(ys) max(ys)]);
else gw.view.tainted = 1;   % don't bother with finesses here, they'd take processor uselessly
end;

% returning gw, stateviews, actionviews, rewardviews, finished

% END gridworld_advance() ==================================================