function v = gridworldview(gw)
%Creates the graphical view over a grid world
%  V = GRIDWORLDVIEW(GW)
%  Creates the GUI view over a grid world.  The view will take care of
 % maintaining and displaying the graphical representation of the
%  grid world.
%
%  Parameters:
%   GW          - the gridworld whose view is being created
%  Returns:
%   V           - the created gridworld view
%
%  See also gridworld, gridworldview_display, gridworldview_isvisible

%  Author:      Lucian Busoniu
%  Version:     1.0
%  History:


% graphical constants
CELL_SIZE = 50;         % size of a square cell in the grid world
PADDING = .25;          % padding inside cell
CIRCLE_DENSITY = 50;    % how many points should circle-emulating polygons have
GOAL_SHAPE = 4;         % how many sides should a goal representation have

% various element colors
COLORS.LINE = [0 0 0];
COLORS.BLANK = [0.85 0.85 0.85];
COLORS.OBSTACLE = [0.5 0.5 0.5];
AGENT_COLORS = [1 0 0; 0 1 0; 0 0 1; 1 1 0; 1 0 1; 0 1 1; 0 0 0; 1 1 1];    % white and black last

width = gw.size(1); height = gw.size(2);
guiw = width * CELL_SIZE; guih = height * CELL_SIZE;

v.width = width; v.height = height;
v.guiw = guiw; v.guih = guih;

% set up the frame
gui = figure(1000 + round(rand * 1000));
set(gui,'Visible', 'off', ...
        'Position', [0 0 guiw guih], ...
        'Color', COLORS.BLANK, ...
        'Name', strcat('Grid World [', gw.name, ']'), ...
        'NumberTitle','off');
clf;
movegui(gui, 'center');

v.gui = gui;

% create the board grids
xlines = linspace(0, guiw, width + 1);
ylines = linspace(0, guih, height + 1);

for i = 1 : width + 1,
   line([xlines(i) xlines(i)], [ylines(1) ylines(height+1)], 'Color', COLORS.LINE);     % horiz
end
for i = 1 : height + 1,
   line([xlines(1) xlines(width+1)], [ylines(i) ylines(i)], 'Color', COLORS.LINE);      % vert
end;

axis([0, guiw, 0, guih]); axis off; hold on;

% determine the radius of the piece, leaving some padding
radius = CELL_SIZE * (1 - PADDING) / 2;

% save the center of the pieces for drawing purposes
xcenter = linspace(CELL_SIZE / 2, guiw - CELL_SIZE / 2, width);
% our representation assumes Y = 0 at the bottom, in Matlab Y = 0 at the top; therefore - flip
ycenter = fliplr( linspace(CELL_SIZE / 2, guih - CELL_SIZE / 2, height) );
xcenters = repmat(xcenter, height, 1);
ycenters = repmat((guih - ycenter)', 1, width);

% agent colors is a number-of-agents by 4 matrix, with the row structure:
% [agent_id R G B]
agentcolors = zeros(0, 4);
% agent color index in predefined colors table
acix = 1;
% handles of polygon fills, will be stored for later display changes
polys = zeros(width, height);

for i = 1 : width,
   for j = 1 : height,
      x = xcenters(j, i) + radius * cos(0 : 2*pi/CIRCLE_DENSITY : 2*pi);
      y = ycenters(j, i) + radius * sin(0 : 2*pi/CIRCLE_DENSITY : 2*pi);
      e = gw.grid(i, j);
      switch e,         % determine cell color based on cell type
          
          case 0,       % empty cell
              polys(i, j) = fill(x, y, COLORS.BLANK);
              set(polys(i, j), 'EdgeColor', COLORS.BLANK);
              
          case -1,      % obstacle cell
              polys(i, j) = fill(x, y, COLORS.OBSTACLE);
              set(polys(i, j), 'EdgeColor', COLORS.OBSTACLE);
              
          otherwise,    % agent or goal
              aid = mod(e, 1000);       % retrieve agent ID
              % get the row where the agent's color was stored, if any
              colorrow = find(agentcolors(:, 1) == aid);
              if isempty(colorrow),   % if none, choose a new color
                  % either from the colors table, if not drained yet
                  if acix <= size(AGENT_COLORS), color = AGENT_COLORS(acix, :);
                  % or some random color
                  else color = [rand rand rand];
                  end;
                  acix = acix + 1;                              % next color
                  agentcolors = [agentcolors; [aid color]];     % store new color
              else                      % color already set, retrieve it
                  color = agentcolors(colorrow, 2:4);
              end;
              % paint goal and agent differently
              if (e >= 1000),       % a goal, diferrent shape
                  x = xcenters(j, i) + radius * cos(0 : 2*pi/GOAL_SHAPE : 2*pi);
                  y = ycenters(j, i) + radius * sin(0 : 2*pi/GOAL_SHAPE : 2*pi);
              end;
              polys(i, j) = fill(x, y, color);
              set(polys(i, j), 'EdgeColor', color);
              
      end;      % END cell type switch
      
  end;          % END for j
end;            % END for i

% store what is needed further in the view structure
v.gui = gui;
v.tainted = 0;
v.colors = COLORS;
v.colors.AGENTS = agentcolors;
v.polys = polys;

% END gridworldview() =================================================================== 