function v = robotsgwview(gw, cfg)
%Creates the graphical view over a grid world
%  V = ROBOTSGWVIEW(GW, CFG)
%  Creates the GUI view over a grid world.  The view will take care of
 % maintaining and displaying the graphical representation of the
%  grid world.
%
%  Parameters:
%   GW          - the robotsgw whose view is being created
%   CFG         - other configuration options. Structure with fields:
%       repres          - either 'abstract' or 'realistic'. In abstract
%           representations, agents are represented by circles, and their goals
%           by crosses. In "realistic", agents are StarWars robots and their
%           goals are color-coded power sockets. Default 'realistic'.
%       showafter       - make world visible only after this number of
%           trials. If Inf, world will be invisible throughout the learnign
%           process. Default Inf.
%       slowdownafter   - similarly, but for slowing down the learning
%           process for visualization. Takes effect only after 'showafter'
%           trials. Default Inf.
%  Returns:
%   V           - the created robotsgw view
%
%  See also robotsgw, robotsgw_display, robotsgwview_isvisible

%  Author:      Lucian Busoniu
%  Version:     2.0
%  History:     2.0 added showafter and slowdownafter options, handling of
%  abstract representation

% default config
DEF_CFG.repres = 'realistic';
DEF_CFG.showafter = Inf;
DEF_CFG.slowdownafter = Inf;
DEF_CFG.slowdownpause = .2;
cfg = checkparams(cfg, DEF_CFG);

% graphical constants
CELL_SIZE = 85;         % size of a square cell in the grid world
PADDING = 4;            % padding inside cell
CONTENT_SIZE = CELL_SIZE - PADDING;
% CIRCLE_DENSITY = 50;    % how many points should circle-emulating polygons have
GOAL_SHAPE = 4;         % how many sides should a goal representation have

% various element colors
COLORS.BACKGROUND = [.85 .85 .85];
COLORS.LINE = [0 0 0];
COLORS.BLANK = [1 1 1];
COLORS.OBSTACLE = [0.5 0.5 0.5];
% AGENT_COLORS = [1 0 0; 0 1 0; 0 0 1; 1 1 0; 1 0 1; 0 1 1; 0 0 0; 1 1 1];    % white and black last

    
width = gw.size(1); height = gw.size(2);
guiw = width * CELL_SIZE; guih = height * CELL_SIZE;

v.width = width; v.height = height;
v.guiw = guiw; v.guih = guih;
v.CONTENT_SIZE = CONTENT_SIZE;

% set up the frame
rand;       % to ensure correct seeding for the algorithms
gui = figure;
set(gui,'Visible', 'off', ...
        'Position', [0 0 guiw guih], ...
        'Color', COLORS.BACKGROUND, ...
        'Name', strcat('Robots World [', gw.name, ']'), ...
        'NumberTitle','off');
clf; hold on;
movegui(gui, 'center');

v.gui = gui; v.visible = 0;

% draw the content pane
bordercoords = [0 0; guiw 0; guiw guih; 0 guih; 0 0];
v.background = fill(bordercoords(:, 1), bordercoords(:, 2), ...
    COLORS.BLANK, 'EdgeColor', COLORS.BLANK);
    
axis([0, guiw, 0, guih]); axis off; hold on;

% save the center of the pieces for drawing purposes
xcenter = linspace(CELL_SIZE / 2, guiw - CELL_SIZE / 2, width);
% our representation assumes Y = 0 at the bottom, in Matlab Y = 0 at the top; therefore - flip
ycenter = fliplr( linspace(CELL_SIZE / 2, guih - CELL_SIZE / 2, height) );
xcenters = repmat(xcenter, height, 1);
ycenters = repmat((guih - ycenter)', 1, width);


% load and create images
% conforming to representation
switch cfg.repres,
    case 'realistic',
        agimgfiles = {'r2d2.jpg', 'g0t0.jpg'};
        goalimgfiles = {'blue-socket.jpg', 'red-socket.jpg'};
    case 'abstract',
        agimgfiles = {'bluedisc.jpg', 'reddisc.jpg'};
        switch gw.goal.type,
            case {1,2},
                % a single goal image, black cross
                goalimgfiles = {'blackcross.jpg'};
            case 3,
                % color-coded crosses
                goalimgfiles = {'bluecross.jpg', 'redcross.jpg'};
        end;
end;

for i = 1:length(agimgfiles),
    img = imread(agimgfiles{i});
    img = imrotate(img, 180);
    AGENT_IMAGES{i} = imresize(img, [CONTENT_SIZE CONTENT_SIZE]);
end;
for i = 1:length(goalimgfiles),
    img = imread(goalimgfiles{i});
    img = imrotate(img, 180);
    GOAL_IMAGES{i} = imresize(img, [CONTENT_SIZE CONTENT_SIZE]);
end;

% Paint static graphic components
for i = 1 : width,
   for j = 1 : height,
       % get graphical centers of cell corresponding to index
      x = xcenters(j, i); y = ycenters(j, i);
      e = gw.grid(i, j);
      switch e,         % determine cell graphics based on cell type
          
          case 0,       % empty cell, no graphics
              
          case -1,      % obstacle cell, completely filled
              polys(i, j) = fill(x + [-CELL_SIZE CELL_SIZE CELL_SIZE -CELL_SIZE]/2, ...
                  y + [-CELL_SIZE -CELL_SIZE CELL_SIZE CELL_SIZE]/2, COLORS.OBSTACLE);
              set(polys(i, j), 'EdgeColor', COLORS.OBSTACLE);
              
          otherwise,    % agent or goal
              if (e > 1000),            % a goal, place image
                  aid = e - 1000;
                  imageindex = 1 + mod(aid - 1, length(GOAL_IMAGES));
                  image('CData', GOAL_IMAGES{imageindex}, ...
                        'XData', x - CONTENT_SIZE/2, 'YData', y - CONTENT_SIZE/2);
              end;
              % agents will be displayed by _display()
              
      end;      % END cell type switch
      
  end;          % END for j
end;            % END for i

% agentimages is a number-of-agents by 2 matrix, with the row structure:
% [agent_id image_handle]
agentimages = zeros(0, 2);
[i, j] = find((gw.grid > 0) & (gw.grid < 1000)); 
for k = 1:length(i),
    aid = gw.grid(i(k), j(k));
    x = xcenters(j(k), i(k)); y = ycenters(j(k), i(k));
    imageindex = 1 + mod(aid - 1, length(AGENT_IMAGES));
    imgh = image('CData', AGENT_IMAGES{imageindex}, ...
      'XData', x - CONTENT_SIZE/2, 'YData', y - CONTENT_SIZE/2);
    agentimages = [agentimages; aid imgh];
end;

% add border last so that it's on top
v.background = line('XData', bordercoords(:, 1), 'YData', bordercoords(:, 2), ...
    'Color', COLORS.OBSTACLE, 'LineWidth', PADDING);



% store what is needed further in the view structure
% v.colors = COLORS;
% v.polys = polys;
v.xcenters = xcenters; v.ycenters = ycenters;
v.agentimages = agentimages;
v.cfg = cfg;
% v.goalimages = goalimages;


% END robotsgwview() =================================================================== 