function gw = rrgw(agents, cfg)
%Creates a 2D grid world
%  GW = rrgw(AGENTS, CFG)
%  Parameters:
%   AGENTS      - the agents in the rrgw, as a (possibly
%               heterogeneous) cell array
%   CFG         - configuration structure with the following fields:
%       name        - the name of the rrgw
%       tasktype    - one of 'coop', 'mixed' for fully cooperative or mixed.
%       size        - the size of the 2D rrgw, a vector [W H]'
%       obstacles   - the coordinates of the obstacles, one obstacle 
%       homebase    - the coordinates of the homebase, [XLL YLL; XUR YUR]'
%       targets     - cell array of targets. Each target is a structure:
%           pos         - position e.g. [X Y]'
%           lockpos     - relative lock positions, e.g. 'sw' for south and west
%           Only one target is considered for cooperative settings. For one target,
%           it can be directly given without wrapping in a cell array.
%       rew         - a structure containing the rewards setup
%           step        - step penalty
%           coll        - collision penalty
%           lock        - lock reward (when locking onto target)
%           sit         - when sitting pinned to target. Default is step
%           pull        - reward for successful pull step while locked on target
%           rip         - penalty for pulling on target uncoordinately (either
%                   alone, or in a different direction than the other agent)
%           home        - reward for successfuly reaching the homebase with
%                   target in tow
%       Rewards can also be given as a vector with the components in this
%       order, [step coll lock sit pull rip home].
%       initpos     - the deterministic initial positions of the
%                   agents. If an empty matrix, at each trial start agents will
%                   be placed in random positions in the world. E.g., [X10 Y10; X20 Y20]'.
%
%  Agents can move one cell in either of the four compass directions,
%  or stay still. Each agent has one action variable available, and its
%  values are coded in the following way:
%   1 - left
%   2 - right
%   3 - up
%   4 - down
%   5 - stay put
%
%
%  See also rrgwview, rrgw_advance, rrgw_destroy,
%      rrgw_obstacles, rrgw_reset

%  Author:      Lucian Busoniu
%  Version:     
%  History:     

% ----------------------------------
% config defaults
DEF.name        = 'default';
DEF.tasktype    = 'coop';
DEF.size = [5 5]';
DEF.obstacles = [];
DEF.homebase = [2 1; 4 2]';
target.pos = [3 4]';
target.lock = 'ew';
DEF.targets = target;
DEF.initpos = [1 1; 5 1]';

DEF.rew.step = -.1;
DEF.rew.coll = -1;
DEF.rew.lock = 5;
DEF.rew.pull = 0;       % pulling is better than normal step
DEF.rew.rip = -1.5;     % typically rip is worse than collision
DEF.rew.home = 50;
    % sit = step in default config


DEF.viewcfg = struct();       % view configuration

% ----------------------------------
% process inputs

% process config
if nargin < 2, cfg = struct(); end; 
cfg = checkparams(cfg, DEF);

% transform targets to structure if just one target given
if isstruct(cfg.targets), cfg.targets = {cfg.targets}; end;

% if numeric rewards array, transform to structure format
if isnumeric(cfg.rew),
    rew = cfg.rew;
    cfg.rew = struct();
    cfg.rew.step    = rew(1);
    cfg.rew.coll    = rew(2);
    cfg.rew.lock    = rew(3);
    cfg.rew.sit     = rew(4);
    cfg.rew.pull    = rew(5);
    cfg.rew.rip     = rew(6);
    cfg.rew.home    = rew(7);
    clear rew;
end;
% by default, step overwrites sit
if ~isfield(cfg.rew, 'sit'),
    cfg.rew.sit = cfg.rew.step;
end;

% transform task type into numeric for easy checking
switch cfg.tasktype,
    case 'coop',
        tt = 1;
        % truncate targets to one
        cfg.targets = {cfg.targets{1}};
%     case 'comp',
%         tt = 2;         % not implemented
    case 'mixed',
        tt = 3;
end;

cfg.tasktype = tt;
clear tt;
gw.cfg = cfg;           % save config
% cfg                     % feedback

% ----------------------------------
% world constants initialization
gw.name = cfg.name;
% action displacements in the 2D space of the grid - left, right, up, down, stay put 
gw.actoffsets = [-1 0; 1 0; 0 -1; 0 1; 0 0]';
% grid world constants
gw.tasktype = 'episodic';
gw.type = 'rrgw';
gw.viewtype = 'rrgwview';

% Grid:
%   0 for empty or homebase (homebase to be checked via coords)
%   -1 for obstacle
%   1 for agent 1
%   2 for agent 2
%   1001 for target 1
%   1002 for target 2
%   101, 102... lock positions, in order
gw.grid = zeros(cfg.size');

% shortcuts to number of agents, number of targets
gw.agn = length(agents);
gw.targn = length(cfg.targets);
if gw.agn > 2,
    error('RobotRescue gridworld only supports up to 2 agents');
end;
% maintain a shortcut to the world state: agent and target positions on columns
gw.state = zeros(2, gw.agn + gw.targn);

% IDs map, only stored, agent IDs within RRGW are fixed
gw.aidmap = zeros(gw.agn, 1);       
for i = 1:gw.agn, gw.aidmap(i) = agents{i}.id; end;


% if deterministic initial positions, init them now
if ~isempty(cfg.initpos),
    for i = 1:gw.agn,
        gw.grid(cfg.initpos(1, i), cfg.initpos(2, i)) = i;
        gw.state(:, i) = cfg.initpos(:, i);
    end;
else gw.state(:, 1:gw.agn) = NaN;     % just a placeholder for state cache
end;


% place targets
% GW fields
% lockflags contains on each position (lock index) the index of the agent which is possibly
% locked there
gw.lockflags = zeros(1, gw.agn);
gw.lockpos = zeros(2, gw.agn);
lockix = 1;     % current lock index (inside collection of locks throughout targets)
targets = {};
for i = 1:length(cfg.targets),
    tpos = cfg.targets{i}.pos; tlock = cfg.targets{i}.lock;
    
    % place target on map
    gw.grid(tpos(1), tpos(2)) = 1000 + i;
    
    % init custom target object
    targets{i}.pos = tpos;
    targets{i}.lockindices = [];
    for j = 1:length(tlock),
        switch tlock(j),
            case 'e',   lockpos = tpos + [1 0]';
            case 'w',   lockpos = tpos + [-1 0]';
            case 's',   lockpos = tpos + [0 -1]';
            case 'n',   lockpos = tpos + [0 1]';
        end;
        % target lock indices
        targets{i}.lockindices(end+1) = lockix;
        % centralized lock positions
        gw.lockpos(:, lockix) = lockpos;
        lockix = lockix + 1; 
    end;
    
    % add to state cache
    gw.state(:, gw.agn + i) = tpos;
end;

% save formatted targets
gw.targets = targets;

% place any specified obstacles
for i = 1 : size(cfg.obstacles, 2),
    gw.grid(cfg.obstacles(1, i), cfg.obstacles(2, i)) = -1;
end;

% save initial state and state cache
gw.grid0 = gw.grid;
gw.state0 = gw.state;
gw.lockpos0 = gw.lockpos;

% ======================================
% create the view
gw.view = rrgwview(gw, cfg.viewcfg);

% Init clocks
gw.k = 0; gw.kt = 0;
gw.trialsk = 0;

% returning gw

% END rrgw()
% =======================================================================