function v = robotsgwview(gw, cfg)
%Creates the graphical view over a grid world
%  V = ROBOTSGWVIEW(GW, CFG)
%  Creates the GUI view over a grid world.  The view will take care of
 % maintaining and displaying the graphical representation of the
%  grid world.
%
%  Parameters:
%   GW          - the robotsgw whose view is being created
%   CFG         - other configuration options. Structure with fields:
%       repres          - either 'abstract' or 'realistic'. In abstract
%           representations, agents are represented by circles, and their goals
%           by ...? TODO. In "realistic", ...? TODO decide if keeping. Default 'abstract'.
%       showafter       - make world visible only after this number of
%           trials. If Inf, world will be invisible throughout the learnign
%           process. Default Inf.
%       slowdownafter   - similarly, but for slowing down the learning
%           process for visualization. Takes effect only after 'showafter'
%           trials. Default Inf.
%  Returns:
%   V           - the created robotsgw view
%
%  See also robotsgw, robotsgw_display, robotsgwview_isvisible

%  Author:      Lucian Busoniu
%  Version:     
%  History:

% default config
DEF_CFG.repres = 'abstract';
DEF_CFG.showafter = Inf;
DEF_CFG.slowdownafter = Inf;
DEF_CFG.slowdownpause = .2;
cfg = checkparams(cfg, DEF_CFG);

% graphical constants
CELL_SIZE = 85;         % size of a square cell in the grid world
PADDING = 4;            % padding inside cell
CONTENT_SIZE = CELL_SIZE - PADDING;
% CIRCLE_DENSITY = 50;    % how many points should circle-emulating polygons have
GOAL_SHAPE = 4;         % how many sides should a goal representation have

% various element colors
COLORS.BACKGROUND = [.85 .85 .85];
COLORS.LINE = [.4 .4 .4];
COLORS.HOMEBASEEDGE = [0 0 0];
COLORS.HOMEBASE = 'none'; %[1 1 1];        % homebase area -- need to make white otherwise doesn't match object background
COLORS.BLANK = [1 1 1];
COLORS.OBSTACLE = [0.5 0.5 0.5];
% AGENT_COLORS = [1 0 0; 0 1 0; 0 0 1; 1 1 0; 1 0 1; 0 1 1; 0 0 0; 1 1 1];    % white and black last

    
width = gw.cfg.size(1); height = gw.cfg.size(2);
guiw = width * CELL_SIZE; guih = height * CELL_SIZE;

v.width = width; v.height = height;
v.guiw = guiw; v.guih = guih;
v.CONTENT_SIZE = CONTENT_SIZE;

% set up the frame
rand;       % to ensure correct seeding for the algorithms
gui = figure;
set(gui,'Visible', 'off', ...
        'Position', [0 0 guiw guih], ...
        'Color', COLORS.BACKGROUND, ...
        'Name', strcat('Robots World [', gw.name, ']'), ...
        'NumberTitle','off');
clf; hold on;
movegui(gui, 'east');

v.gui = gui; v.visible = 0;

% draw the content pane
bordercoords = [0 0; guiw 0; guiw guih; 0 guih; 0 0];
v.background = fill(bordercoords(:, 1), bordercoords(:, 2), ...
    COLORS.BLANK, 'EdgeColor', COLORS.BLANK);
    
axis([0, guiw, 0, guih]); axis off; hold on;

% save the center of the pieces for drawing purposes
xcenter = linspace(CELL_SIZE / 2, guiw - CELL_SIZE / 2, width);
% our representation assumes Y = 0 at the bottom, in Matlab Y = 0 at the top; therefore - flip
ycenter = fliplr( linspace(CELL_SIZE / 2, guih - CELL_SIZE / 2, height) );
xcenters = repmat(xcenter, height, 1);
ycenters = repmat((guih - ycenter)', 1, width);


% load and create images
% conforming to representation
switch cfg.repres,
    case 'realistic',
        % for now, the same as abstract
        agimgfiles = {'bluedisc.jpg', 'greendisc.jpg'};
        targetimgfiles = {'redline.png', 'redline.png'};
    case 'abstract',
%         agimgfiles = {'bluedisc.jpg', 'greendisc.jpg'};
        % 2009-05-14: changed to use numbered discs
        agimgfiles = {'bluedisc_numbered.jpg', 'greendisc_numbered.jpg'};
        % same image for all victims
        targetimgfiles = {'redline.png', 'redline.png'};
end;

for i = 1:length(agimgfiles),
    img = imread(agimgfiles{i});
    img = imrotate(img, 180);
    AGENT_IMAGES{i} = imresize(img, [CONTENT_SIZE CONTENT_SIZE]);
end;
for i = 1:length(targetimgfiles),
    img = imread(targetimgfiles{i});
    img = imrotate(img, 180);
    GOAL_IMAGES{i} = imresize(img, [CONTENT_SIZE CONTENT_SIZE]);
end;

% Paint graphic components

% === paint grid first, needs to go under everything
% create the board grids
xlines = linspace(0, guiw, width + 1);
ylines = linspace(0, guih, height + 1);

for i = 1 : width + 1,
   line([xlines(i) xlines(i)], [ylines(1) ylines(height+1)], 'LineStyle', ':', 'Color', COLORS.LINE);     % horiz
end;
for i = 1 : height + 1,
   line([xlines(1) xlines(width+1)], [ylines(i) ylines(i)], 'LineStyle', ':', 'Color', COLORS.LINE);      % vert
end;
% === paint homebase

% retrieve lowerleft and upperright coords
hll = gw.cfg.homebase(:, 1); hur = gw.cfg.homebase(:, 2);
% transform into actual figure coords
hll = [xcenters(hll(2), hll(1)), ycenters(hll(2), hll(1))];
hur = [xcenters(hur(2), hur(1)), ycenters(hur(2), hur(1))];
py = fill([hll(1) - CELL_SIZE/2, hll(1) - CELL_SIZE/2, hur(1) + CELL_SIZE/2, hur(1) + CELL_SIZE/2], ...
    [hll(2) - CELL_SIZE/2, hur(2) + CELL_SIZE/2, hur(2) + CELL_SIZE/2, hll(2) - CELL_SIZE/2, ], 'w');
% above color is just a placeholder, we actually set the color here
set(py, 'EdgeColor', COLORS.HOMEBASEEDGE, 'LineStyle', '--', 'LineWidth', 2, 'FaceColor', COLORS.HOMEBASE);

% === paint agents

agentimages = zeros(1, gw.agn);
for aix = 1:gw.agn,
    [i, j] = find(gw.grid == aix);
    % if agent not yet on grid, place it in the first emtpy cell
    if isempty(i), [i, j] = find(gw.grid == 0, 1, 'first'); end;
    x = xcenters(j, i); y = ycenters(j, i);
    imageindex = 1 + mod(aix - 1, length(AGENT_IMAGES));
    agentimages(aix) = image('CData', AGENT_IMAGES{imageindex}, ...
      'XData', x - CONTENT_SIZE/2, 'YData', y - CONTENT_SIZE/2);
end;

% === paint obstacles, targets
targetimages = 0*agentimages;

for i = 1 : width,
   for j = 1 : height,
       % get graphical centers of cell corresponding to index
      x = xcenters(j, i); y = ycenters(j, i);
      e = gw.grid(i, j);
      switch e,         % determine cell graphics based on cell type
          
          case 0,       % empty cell, no graphics
              
          case -1,      % obstacle cell, completely filled
              % we don't need to remember this image
              py = fill(x + [-CELL_SIZE CELL_SIZE CELL_SIZE -CELL_SIZE]/2, ...
                  y + [-CELL_SIZE -CELL_SIZE CELL_SIZE CELL_SIZE]/2, COLORS.OBSTACLE);
              set(py, 'EdgeColor', COLORS.OBSTACLE);
              
          otherwise,    % agent or target
              if (e > 1000),            % a target, place image
                  aix = e - 1000;
                  imageindex = 1 + mod(aix - 1, length(GOAL_IMAGES));
                  targetimages(aix) = image('CData', GOAL_IMAGES{imageindex}, ...
                        'XData', x - CONTENT_SIZE/2, 'YData', y - CONTENT_SIZE/2);
              end;
              % agents will be displayed by _display()
              
      end;      % END cell type switch
      
  end;          % END for j
end;            % END for i

% add border last so that it's on top
v.background = line('XData', bordercoords(:, 1), 'YData', bordercoords(:, 2), ...
    'Color', COLORS.OBSTACLE, 'LineWidth', PADDING);

% store what is needed further in the view structure
% v.colors = COLORS;
% v.polys = polys;
v.xcenters = xcenters; v.ycenters = ycenters;
v.agentimages = agentimages;
v.targetimages = targetimages;
v.cfg = cfg;


% END robotsgwview() =================================================================== 